/* Copyright © 2016-2021 Ganchrow Scientific, SA all rights reserved */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const os = require("os");
const dnsSync = require("dns-sync");
function getNetworkIP() {
    try {
        return getNetworkIPs()[0];
    }
    catch (e) {
        // assume network is turned off
        return '';
    }
}
exports.getNetworkIP = getNetworkIP;
function getNetworkIPs() {
    try {
        let ifaces = os.networkInterfaces();
        return Object.keys(ifaces)
            .map(i => ifaces[i]
            .filter(x => x.family === 'IPv4' && !x.internal)[0])
            .filter(f => !!f)
            .map(iface => iface.address);
    }
    catch (e) {
        // assume network is turned off
        return [''];
    }
}
exports.getNetworkIPs = getNetworkIPs;
function isExactMatchWrap(item) {
    return (otherItem) => item === otherItem;
}
function possibleLocalHostNames() {
    return ['localhost', '127.0.0.1', os.hostname(), ...getNetworkIPs()];
}
function isRemoteHost(host) {
    return !(!host || possibleLocalHostNames().some(isExactMatchWrap(host)));
}
exports.isRemoteHost = isRemoteHost;
function isLocalHost(host) {
    const ip = dnsSync.resolve(host);
    return ip ? possibleLocalHostNames().some(isExactMatchWrap(ip)) :
        !isRemoteHost(host);
}
exports.isLocalHost = isLocalHost;
//# sourceMappingURL=hostName.js.map