/* Copyright © 2025 Ganchrow Scientific, SA all rights reserved */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const simple_statistics_1 = require("simple-statistics");
class MetricAggregator {
    constructor(flushPeriodMs = 60000, flushSize = 1000) {
        this.flushPeriodMs = flushPeriodMs;
        this.flushSize = flushSize;
        this.items = [];
        this.lastFlushTs = Date.now();
        if (this.flushPeriodMs < 1) {
            throw new Error('Flush period must be greater than 0; got ' + this.flushPeriodMs);
        }
        if (this.flushSize < 1) {
            throw new Error('Flush size must be greater than 0; got ' + this.flushSize);
        }
    }
    add(value, nowTs = Date.now()) {
        this.items.push(value);
        if (this.items.length >= this.flushSize
            || nowTs - this.lastFlushTs >= this.flushPeriodMs) {
            return this.flush(nowTs);
        }
        return null;
    }
    flush(nowTs = Date.now()) {
        const count = this.items.length;
        const periodMs = nowTs - this.lastFlushTs;
        const from = (new Date(this.lastFlushTs)).toISOString();
        const to = (new Date(nowTs)).toISOString();
        this.lastFlushTs = nowTs;
        if (count < 1) {
            return {
                from,
                to,
                periodMs,
                count,
            };
        }
        const items = this.items;
        this.items = [];
        return {
            from,
            to,
            periodMs,
            count,
            min: simple_statistics_1.min(items),
            max: simple_statistics_1.max(items),
            mean: simple_statistics_1.mean(items),
        };
    }
}
exports.MetricAggregator = MetricAggregator;
//# sourceMappingURL=metricAggregator.js.map